@@ -1,10 +1,9 @@
 /*
- * Copyright 2005-2008 Gentoo Foundation
+ * Copyright 2005-2014 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/libq/atom_explode.c,v 1.27 2013/04/29 04:38:16 vapier Exp $
  *
  * Copyright 2005-2008 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2008 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2014 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 typedef enum { VER_ALPHA=0, VER_BETA, VER_PRE, VER_RC, VER_NORM, VER_P } atom_suffixes;
@@ -38,7 +37,7 @@ typedef struct {
 	char letter;
 	atom_suffix *suffixes;
 	char *PV, *PVR;
-	char *P, *SLOT;
+	char *P, *SLOT, *REPO;
 } depend_atom;
 
 #ifdef _USE_CACHE
@@ -116,6 +115,12 @@ depend_atom *atom_explode(const char *atom)
 	if ((ptr = strstr(ret->CATEGORY, ".ebuild")) != NULL)
 		*ptr = '\0';
 
+	/* chip off the trailing [::REPO] as needed */
+	if ((ptr = strstr(ret->CATEGORY, "::")) != NULL) {
+		ret->REPO = ptr + 2;
+		*ptr = '\0';
+	}
+
 	/* chip off the trailing [:SLOT] as needed */
 	if ((ptr = strrchr(ret->CATEGORY, ':')) != NULL) {
 		ret->SLOT = ptr + 1;
