@@ -1,10 +1,9 @@
 /*
- * Copyright 2005-2010 Gentoo Foundation
+ * Copyright 2005-2014 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qmerge.c,v 1.130 2013/05/09 05:28:11 vapier Exp $
  *
  * Copyright 2005-2010 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2010 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2014 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 #ifdef APPLET_qmerge
@@ -48,9 +47,9 @@ static struct option const qmerge_long_opts[] = {
 	{"yes",     no_argument, NULL, 'y'},
 	{"nodeps",  no_argument, NULL, 'O'},
 	{"nomd5",   no_argument, NULL, '5'},
+	{"debug",   no_argument, NULL, 128},
 	COMMON_LONG_OPTS
 };
-
 static const char * const qmerge_opts_help[] = {
 	"Fetch package and newest Packages metadata",
 	"Fetch package (skipping Packages)",
@@ -62,10 +61,9 @@ static const char * const qmerge_opts_help[] = {
 	"Don't prompt before overwriting",
 	"Don't merge dependencies",
 	"Don't verify MD5 digest of files",
+	"Run shell funcs with `set -x`",
 	COMMON_OPTS_HELP
 };
-
-static const char qmerge_rcsid[] = "$Id: qmerge.c,v 1.130 2013/05/09 05:28:11 vapier Exp $";
 #define qmerge_usage(ret) usage(ret, QMERGE_FLAGS, qmerge_long_opts, qmerge_opts_help, lookup_applet_idx("qmerge"))
 
 char search_pkgs = 0;
@@ -77,6 +75,7 @@ char follow_rdepends = 1;
 char nomd5 = 0;
 char qmerge_strict = 0;
 char update_only = 0;
+bool debug = false;
 const char Packages[] = "Packages";
 
 /*
@@ -113,16 +112,15 @@ _q_static char *find_binpkg(const char *);
 
 _q_static void fetch(const char *destdir, const char *src)
 {
-	char buf[BUFSIZ];
-
 	if (!binhost[0])
-		errf("PORTAGE_BINHOST= does not appear to be valid");
+		return;
 
 	fflush(stdout);
 	fflush(stderr);
 
 #if 0
 	if (getenv("FETCHCOMMAND") != NULL) {
+		char buf[BUFSIZ];
 		snprintf(buf, sizeof(buf), "(export DISTDIR='%s' URI='%s/%s'; %s)",
 			destdir, binhost, src, getenv("FETCHCOMMAND"));
 		xsystem(buf);
@@ -131,6 +129,9 @@ _q_static void fetch(const char *destdir, const char *src)
 	{
 		pid_t p;
 		int status;
+		char *path;
+
+		xasprintf(&path, "%s/%s", binhost, src);
 
 		char prog[] = "wget";
 		char argv_c[] = "-c";
@@ -142,18 +143,22 @@ _q_static void fetch(const char *destdir, const char *src)
 			argv_c,
 			argv_P,
 			argv_dir,
-			buf,
+			path,
 			quiet ? argv_q : NULL,
 			NULL,
 		};
 		if (!(force_download || install) && pretend)
 			strcpy(prog, "echo");
-		snprintf(buf, sizeof(buf), "%s/%s", binhost, src);
 
 		p = vfork();
-		if (p == 0)
+		switch (p) {
+		case 0:
 			_exit(execvp(prog, argv));
+		case -1:
+			errp("vfork failed");
+		}
 
+		free(path);
 		free(argv_dir);
 
 		waitpid(p, &status, 0);
@@ -237,7 +242,7 @@ _q_static char *best_version(const char *catname, const char *pkgname)
 	/* Make sure these dirs exist before we try walking them */
 	switch (vdb_check) {
 	case 1: {
-		int fd = open(portroot, O_RDONLY|O_CLOEXEC);
+		int fd = open(portroot, O_RDONLY|O_CLOEXEC|O_PATH);
 		if (fd >= 0) {
 			/* skip leading slash */
 			vdb_check = faccessat(fd, portvdb + 1, X_OK, 0);
@@ -352,6 +357,7 @@ qprint_tree_node(int level, const depend_atom *atom, const struct pkg_t *pkg)
 
 	char install_ver[126] = "";
 	char c = 'N';
+	const char *color;
 
 	if (!pretend)
 		return 0;
@@ -380,10 +386,13 @@ qprint_tree_node(int level, const depend_atom *atom, const struct pkg_t *pkg)
 			return c;
 		if ((c == 'R' || c == 'D') && update_only && level)
 			return c;
-		if (c == 'R')
-			snprintf(buf, sizeof(buf), "%s%c%s", YELLOW, c, NORM);
-		if (c == 'U' || c == 'D')
-			snprintf(buf, sizeof(buf), "%s%c%s", BLUE, c, NORM);
+		switch (c) {
+		case 'R': color = YELLOW; break;
+		case 'U': color = BLUE; break;
+		case 'D': color = DKBLUE; break;
+		default: color = RED; break;
+		}
+		snprintf(buf, sizeof(buf), "%s%c%s", color, c, NORM);
 #if 0
 		if (level) {
 			switch (c) {
@@ -431,7 +440,7 @@ pkg_run_func(const char *vdb_path, const char *phases, const char *func, const c
 		"debug-print-function() { :; }\n"
 		"debug-print-section() { :; }\n"
 		/* Not quite right */
-		"has_version() { qlist -ICq -e '$1' >/dev/null; }\n"
+		"has_version() { [ -n \"$(qlist -ICqe \"$1\")\" ]; }\n"
 		/* best_version() */
 		"use() { useq \"$@\"; }\n"
 		"usex() { useq \"$1\" && echo \"${2-yes}$4\" || echo \"${3-no}$5\"; }\n"
@@ -447,11 +456,13 @@ pkg_run_func(const char *vdb_path, const char *phases, const char *func, const c
 		"die() { eerror \"$@\"; exit 1; }\n"
 		"ebegin() { printf ' * %%b ...' \"$*\"; }\n"
 		"eend() { local r=${1:-$?}; [ $# -gt 0 ] && shift; [ $r -eq 0 ] && echo ' [ ok ]' || echo \" $* \"'[ !! ]'; return $r; }\n"
+		"emake() { ${MAKE:-make} ${MAKEOPTS} \"$@\"; }\n"
 		/* Unpack the env if need be */
 		"[ -e '%1$s/environment' ] || { bzip2 -dc '%1$s/environment.bz2' > '%1$s/environment' || exit 1; }\n"
 		/* Load the main env */
 		". '%1$s/environment'\n"
 		/* Reload env vars that matter to us */
+		"FILESDIR=/.does/not/exist/anywhere\n"
 		"MERGE_TYPE=binary\n"
 		"ROOT='%4$s'\n"
 		"EROOT=\"${EPREFIX%%/}/${ROOT#/}\"\n"
@@ -459,10 +470,16 @@ pkg_run_func(const char *vdb_path, const char *phases, const char *func, const c
 		"ED=\"${EPREFIX%%/}/${D#/}\"\n"
 		"T='%6$s'\n"
 		/* Finally run the func */
-		"%2$s\n"
+		"%7$s%2$s\n"
 		/* Ignore func return values (not exit values) */
 		":",
-		vdb_path, func, phase, portroot, D, T);
+		/*1*/ vdb_path,
+		/*2*/ func,
+		/*3*/ phase,
+		/*4*/ portroot,
+		/*5*/ D,
+		/*6*/ T,
+		/*7*/ debug ? "set -x;" : "");
 	xsystembash(script);
 	free(script);
 }
@@ -483,10 +500,11 @@ merge_tree_at(int fd_src, const char *src, int fd_dst, const char *dst,
 	ret = -1;
 
 	/* Get handles to these subdirs */
+	/* Cannot use O_PATH as we want to use fdopendir() */
 	subfd_src = openat(fd_src, src, O_RDONLY|O_CLOEXEC);
 	if (subfd_src < 0)
 		return ret;
-	subfd_dst = openat(fd_dst, dst, O_RDONLY|O_CLOEXEC);
+	subfd_dst = openat(fd_dst, dst, O_RDONLY|O_CLOEXEC|O_PATH);
 	if (subfd_dst < 0) {
 		close(subfd_src);
 		return ret;
@@ -736,7 +754,9 @@ pkg_merge(int level, const depend_atom *atom, const struct pkg_t *pkg)
 {
 	queue *objs;
 	FILE *fp, *contents;
-	char buf[1024], phases[128];
+	static char *phases;
+	static size_t phases_len;
+	char buf[1024];
 	char *tbz2, *p, *D, *T;
 	int i;
 	char **ARGV;
@@ -796,25 +816,12 @@ pkg_merge(int level, const depend_atom *atom, const struct pkg_t *pkg)
 						/* warn("newname = %s", name); */
 					}
 					if ((subatom = atom_explode(name)) != NULL) {
-						char *dep;
 						struct pkg_t *subpkg;
 						char *resolved = NULL;
 
-						dep = find_binpkg(name);
+						resolved = find_binpkg(name);
 
-						if (strncmp(name, "virtual/", 8) == 0) {
-							if (virtuals == NULL)
-								virtuals = resolve_virtuals();
-							resolved = find_binpkg(virtual(name, virtuals));
-							if (resolved == NULL || !strlen(resolved))
-								resolved = find_binpkg(name);
-						} else
-							resolved = NULL;
-
-						if (resolved == NULL)
-							resolved = dep;
-
-						IF_DEBUG(fprintf(stderr, "+Atom: argv0(%s) dep(%s) resolved(%s)\n", name, dep, resolved));
+						IF_DEBUG(fprintf(stderr, "+Atom: argv0(%s) resolved(%s)\n", name, resolved));
 
 						if (strlen(resolved) < 1) {
 							warn("Cant find a binpkg for %s from rdepend(%s)", name, pkg->RDEPEND);
@@ -886,7 +893,7 @@ pkg_merge(int level, const depend_atom *atom, const struct pkg_t *pkg)
 	xsystem(buf);
 	fflush(stdout);
 
-	eat_file("vdb/DEFINED_PHASES", phases, sizeof(phases));
+	eat_file("vdb/DEFINED_PHASES", &phases, &phases_len);
 	pkg_run_func("vdb", phases, "pkg_pretend", D, T);
 	pkg_run_func("vdb", phases, "pkg_setup", D, T);
 	pkg_run_func("vdb", phases, "pkg_preinst", D, T);
@@ -1014,7 +1021,9 @@ _q_static int
 pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 {
 	size_t buflen;
-	char *buf, *vdb_path, *T, phases[128];
+	static char *phases;
+	static size_t phases_len;
+	char *buf, *vdb_path, *T;
 	FILE *fp;
 	int ret, fd, vdb_fd, portroot_fd;
 	int cp_argc, cpm_argc;
@@ -1038,7 +1047,7 @@ pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 		return 0;
 
 	/* Get a handle to the root to play with */
-	portroot_fd = open(portroot, O_RDONLY | O_CLOEXEC);
+	portroot_fd = open(portroot, O_RDONLY|O_CLOEXEC|O_PATH);
 	if (portroot_fd == -1) {
 		warnp("unable to read %s", portroot);
 		goto done;
@@ -1048,7 +1057,7 @@ pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 	/* Note: This vdb_path must be absolute since we use it in pkg_run_func() */
 	xasprintf(&vdb_path, "%s%s/%s/%s/", portroot, portvdb, cat, pkgname);
 	xasprintf(&T, "%stemp", vdb_path);
-	vdb_fd = openat(portroot_fd, vdb_path, O_RDONLY | O_CLOEXEC);
+	vdb_fd = openat(portroot_fd, vdb_path, O_RDONLY|O_CLOEXEC|O_PATH);
 	if (vdb_fd == -1) {
 		warnp("unable to read %s", vdb_path);
 		goto done;
@@ -1056,7 +1065,7 @@ pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 
 	/* First execute the pkg_prerm step */
 	if (!pretend) {
-		eat_file_at(vdb_fd, "DEFINED_PHASES", phases, sizeof(phases));
+		eat_file_at(vdb_fd, "DEFINED_PHASES", &phases, &phases_len);
 		mkdir_p(T, 0755);
 		pkg_run_func(vdb_path, phases, "pkg_prerm", T, T);
 	}
@@ -1157,8 +1166,11 @@ pkg_unmerge(const char *cat, const char *pkgname, queue *keep)
 		if (!keep || q) {
 			char *p;
 
-			if (unlinkat(portroot_fd, e->name + 1, 0))
-				errp("could not unlink: %s%s", portroot, e->name + 1);
+			if (unlinkat(portroot_fd, e->name + 1, 0)) {
+				/* If a file was already deleted, ignore the error */
+				if (errno != ENOENT)
+					errp("could not unlink: %s%s", portroot, e->name + 1);
+			}
 
 			p = strrchr(e->name, '/');
 			if (p) {
@@ -1366,7 +1378,7 @@ pkg_fetch(int level, const depend_atom *atom, const struct pkg_t *pkg)
 	}
 	if (access(buf, R_OK) == 0) {
 		if (!pkg->SHA1[0] && !pkg->MD5[0]) {
-			warn("No checksum data for %s", buf);
+			warn("No checksum data for %s (try `emaint binhost --fix`)", buf);
 			return;
 		} else {
 			if (pkg_verify_checksums(buf, pkg, atom, qmerge_strict, !quiet) == 0) {
@@ -1421,7 +1433,7 @@ print_Pkg(int full, struct pkg_t *pkg)
 	printf("%s%s/%s%s%s%s%s%s\n", BOLD, pkg->CATEGORY, BLUE, pkg->PF, NORM,
 		!quiet ? " [" : "",
 		!quiet ? make_human_readable_str(pkg->SIZE, 1, KILOBYTE) : "",
-		!quiet ? "KB]" : "");
+		!quiet ? "KiB]" : "");
 
 	if (full == 0)
 		return;
@@ -1508,20 +1520,51 @@ _q_static FILE *
 open_binpkg_index(void)
 {
 	FILE *fp;
-	char buf[BUFSIZ];
+	char *path;
 
-	snprintf(buf, sizeof(buf), "%s/portage/%s", port_tmpdir, Packages);
-	fp = fopen(buf, "r");
+	xasprintf(&path, "%s/portage/%s", port_tmpdir, Packages);
+	fp = fopen(path, "r");
 	if (fp)
-		return fp;
+		goto done;
+	free(path);
 
-	snprintf(buf, sizeof(buf), "%s/%s", pkgdir, Packages);
-	fp = fopen(buf, "r");
+	xasprintf(&path, "%s/%s", pkgdir, Packages);
+	fp = fopen(path, "r");
 	if (fp)
-		return fp;
+		goto done;
 
-	errp("Unable to open package file %s in %s/portage or %s",
+	/* This is normal when installing from local repo only. */
+	warnp("Unable to open package file %s in %s/portage or %s",
 		Packages, port_tmpdir, pkgdir);
+	warn("Attempting to manually regen via `emaint binhost`");
+
+	pid_t p;
+	int status;
+
+	char argv_emaint[] = "emaint";
+	char argv_binhost[] = "binhost";
+	char argv_fix[] = "--fix";
+	char *argv[] = {
+		argv_emaint,
+		argv_binhost,
+		argv_fix,
+		NULL,
+	};
+
+	p = vfork();
+	switch (p) {
+	case 0:
+		_exit(execvp(argv[0], argv));
+	case -1:
+		errp("vfork failed");
+	}
+	waitpid(p, &status, 0);
+
+	fp = fopen(path, "r");
+
+ done:
+	free(path);
+	return fp;
 }
 
 _q_static struct pkg_t *
@@ -1800,7 +1843,7 @@ parse_packages(queue *todo)
 	free(buf);
 	fclose(fp);
 
-	return 0;
+	return EXIT_SUCCESS;
 }
 
 _q_static queue *
@@ -1918,6 +1961,7 @@ int qmerge_main(int argc, char **argv)
 			case 'y': interactive = 0; break;
 			case 'O': follow_rdepends = 0; break;
 			case '5': nomd5 = 1; break;
+			case 128: debug = true; break;
 			COMMON_GETOPTS_CASES(qmerge)
 		}
 	}
